(*
 * ocamlsdl - Objective Caml interface for the SDL library
 * Copyright (C) 1999 Jean-Christophe FILLIATRE
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License version 2, as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU Library General Public License version 2 for more details
 * (enclosed in the file LGPL).
 *)

(* $Id: sdl.mli,v 1.13 2002/12/10 16:36:14 filliatr Exp $ *)

exception SDL_failure of string


(* Initialization. *)

type init_flag =
  | TIMER
  | AUDIO
  | VIDEO
  | CDROM
  | JOYSTICK
  | NOPARACHUTE        (* Don't catch fatal signals *)
  | EVENTTHREAD
  | EVERYTHING

val init : init_flag list -> unit
val quit : unit -> unit


(* Video. *)

module Video : sig


  type video_flag =
    | SWSURFACE   (* Surface is in system memory *)
    | HWSURFACE   (* Surface is in video memory *)
    | ANYFORMAT   (* Allow any video pixel format *)
    | HWPALETTE   (* Surface has exclusive palette *)
    | DOUBLEBUF   (* Set up double-buffered video mode *)
    | FULLSCREEN  (* Surface is a full screen display *)
    | HWACCEL     (* Blit uses hardware acceleration *)
    | SRCCOLORKEY (* Blit uses a source color key *)
    | RLEACCEL    (* Colorkey blit is RLE accelerated *)
    | SRCALPHA    (* Blit uses source alpha blending *)
    | SRCCLIPPING (* Blit uses source clipping *)
    | OPENGL

  type surface

  val surface_width : surface -> int
  val surface_height : surface -> int
  val surface_flags : surface -> video_flag list
  val surface_bpp : surface -> int
  val surface_rmask : surface -> int
  val surface_gmask : surface -> int
  val surface_bmask : surface -> int
  val surface_amask : surface -> int

  val free_surface : surface -> unit

  val must_lock : surface -> bool
  val lock_surface : surface -> unit
  val unlock_surface : surface -> unit

  val video_mode_ok : int -> int -> int -> video_flag list -> bool
	 (* [video_mode_ok width height bpp flags] *)
  val set_video_mode : int -> int -> int -> video_flag list -> surface
	 (* [set_video_mode width height bpp flags] *)

  val alloc_surface : 
    video_flag list -> int -> int -> int -> int -> int -> int -> int
	-> surface
	(* [alloc_surface flags w h depth rmask gmask bmask amask] *)
  
  val load_bmp : string -> surface
  val save_bmp : surface -> string -> unit
      
  val set_color_key : surface -> video_flag list -> int -> unit
      (* [set_color_key surf flag key] *)
  val set_alpha : surface -> int -> int -> unit
      (* [set_alpha surf flag alpha] *)
(*
  val set_clipping : surface -> int -> int -> int -> int -> unit
      (* [set_clipping surf top left bottom right] *)
  val disable_clipping : surface -> unit
*)
  val display_format : surface -> surface

  val map_rgb : surface -> int -> int -> int -> int

  type rect = {
    mutable rect_x : int;
    mutable rect_y : int;
    mutable rect_w : int;
    mutable rect_h : int }

  val fill_surface : surface -> int -> unit
  val fill_rect : surface -> rect -> int -> unit

  val update_surface : surface -> unit
  val update_rect : surface -> int -> int -> int -> int -> unit
  val update_rects : surface -> int -> rect array -> unit

  val flip : surface -> unit

  val blit_surface : surface -> rect option -> surface -> rect option -> unit
      (* beware: rectangles are modified if clipped! *)

  type color = { 
    red : int;     (* 0..255 *)
    green : int;   (* 0..255 *)
    blue : int     (* 0..255 *)
  }
		 
  val set_colors : surface -> color array -> int -> int -> bool
      (* [set_colors surf colors firstcolor ncolors] *)

  val show_cursor : bool -> unit

  external string_of_pixels : surface -> string = "sdlstub_string_of_pixels"

end


(* Open GL support *)

module GL : sig

  external swap_buffers : unit -> unit = "sdlstub_GL_swap_buffers"

  external load_bmp : string -> Video.surface = "sdlstub_GL_load_bmp"

end


(* Events. *)

module Event : sig

  type keysym = { 
    scancode : int;
    key : int;
    modif : int;
    unicode : int
  }

  type keystate = Pressed | Released

  type keyboard = {
    key_state : keystate;
    key_sym : keysym;
  }

  type event = 
    | Active 
    | KeyDown of keyboard
    | KeyUp of keyboard
    | MouseMotion
    | MouseButtonDown
    | MouseButtonUp
    | JoyMotion
    | JoyButtonDown
    | JoyButtonUp
    | Quit
    | SysWm
    | UnknowEvent

  val poll_event : unit -> event option

  val wait_event : unit -> event
  
end


(* Timer. *)

module Timer : sig

  val get_ticks : unit -> int

  val delay : int -> unit

end


(* Drawing. *)

module Draw : sig

  val put_pixel : Video.surface -> int -> int -> int -> unit
      
  val get_pixel : Video.surface -> int -> int -> int

end


(* low-level audio. See also High-level audio in mixer.mli *)

module Audio : sig

(*
  val open_audio : unit -> unit
*)

  val close_audio : unit -> unit

  val unlock_audio : unit -> unit

end

(*i
module Ttf : sig 
  
  val init : unit -> unit

  exception SDLttf_exception of string

  type font

  val open_font : string -> int -> font
  val close_font : font -> unit
  val font_height : font -> int
  val render_text : font -> string -> (int * int * int) -> (int * int * int) -> Video.surface
      
(* Return a function to print strings, and another to clean up printer *)
val make_printer : font -> (int * int * int) ->
  (Video.surface -> int -> int -> string -> unit) * (unit -> unit)

end
i*)
