(* Test program for ocamlsdl. *)

open Sdl
open Video
open Event

let rec remove x l = 
  match l with
      [] -> []
    | y::l -> if x=y then l else y::(remove x l)
;;

let parse_cmdline () =
  let flags = ref [SWSURFACE ; ANYFORMAT] in
  let bpp = ref 0 in
  let nsprites = ref 3 in
  Arg.parse 
      [ "-bpp", Arg.Int (fun n -> 
			   bpp := n; 
			   flags := remove ANYFORMAT !flags),
	" n   sets bpp";
       	"-fullscreen", Arg.Unit (fun () -> 
				   flags := FULLSCREEN :: !flags),
	"  sets full screen mode" ]
      (fun s -> nsprites := int_of_string s)
      "Usage: test [-bpp N] [-hw] [-flip] [-fullscreen] [numsprites]";
  !flags,!bpp,!nsprites

let load_sprite file =
  let tmp = load_bmp file in
  set_color_key tmp [SRCCOLORKEY ; RLEACCEL] 0;
  let spr = display_format tmp in
  free_surface tmp;
  spr

let init () =
  try
    Sdl.init [VIDEO];
    show_cursor false
  with SDL_failure m ->
    Printf.eprintf "SDL init failure: %s\n" m;
    exit 1

type sprite = {
  mutable pos_x : int;
  mutable pos_y : int;
  mutable vit_x : int;
  mutable vit_y : int
}

let max_speed = 1

let screen_width = 640
let screen_height = 480

let new_sprite screen =
  let vx = ref 0 and vy = ref 0 in
  while !vx = 0 && !vy = 0 do
    vx := (Random.int (max_speed * 2 + 1)) - max_speed;
    vy := (Random.int (max_speed * 2 + 1)) - max_speed
  done;
  { pos_x = Random.int screen_width;
    pos_y = Random.int screen_height;
    vit_x = !vx;
    vit_y = !vy }

let move_sprites sprites =
  for i = 0 to Array.length sprites - 1 do
    let s = sprites.(i) in
    s.pos_x <- s.pos_x + s.vit_x;
    if s.pos_x < 0 || s.pos_x > screen_width then begin
      s.vit_x <- -s.vit_x;
      s.pos_x <- s.pos_x + s.vit_x
    end;
    s.pos_y <- s.pos_y + s.vit_y;
    if s.pos_y < 0 || s.pos_y > screen_height then begin
      s.vit_y <- -s.vit_y;
      s.pos_y <- s.pos_y + s.vit_y
    end
  done
      
let main () =
  init ();
  let flags,bpp,nsprites = parse_cmdline () in
  Printf.printf "setting video mode..."; flush stdout;
  let screen = set_video_mode screen_width screen_height bpp flags in
  Printf.printf "ok\n"; flush stdout;
  Printf.printf "screen size: %d x %d  bpp = %d\n" (surface_width screen) 
    (surface_height screen) (surface_bpp screen);
  let double_buffer = List.mem DOUBLEBUF (surface_flags screen) in
  Printf.printf "double buffer: %b\n" double_buffer;
  let sprite = load_sprite "icon.bmp" in
  let sprite_w = surface_width sprite in
  let sprite_h = surface_height sprite in
  Printf.printf "sprite size: %d x %d\n" sprite_w sprite_h;
  flush stdout;
  let sprites = Array.init nsprites (fun _ -> new_sprite screen) in
  (*
  for i = 0 to nsprites - 1 do
    Printf.printf "sprite # %d : %d %d %d %d\n" i sprites.(i).pos_x
      sprites.(i).pos_y sprites.(i).vit_x sprites.(i).vit_y 
  done;
  flush stdout;
  *)
  let bgcolor = map_rgb screen 0 0 0 in
  let before = Timer.get_ticks () in
  let finished = ref false in
  let frames = ref 0 in
  let updates = 
    Array.init nsprites 
      (fun _ -> { rect_x = 0; rect_y = 0; rect_w = 0; rect_h = 0 }) 
  in
  while not !finished do
    incr frames;
    begin match poll_event () with
      | Some (KeyDown _ | Quit) -> finished := true
      | _ -> ()
    end;
    fill_surface screen bgcolor;
    move_sprites sprites;
    for i = 0 to nsprites - 1 do
      updates.(i).rect_x <- sprites.(i).pos_x;
      updates.(i).rect_y <- sprites.(i).pos_y;
      updates.(i).rect_w <- sprite_w;
      updates.(i).rect_h <- sprite_h;
      blit_surface sprite None screen (Some updates.(i))
    done;
    update_rects screen nsprites updates;
    (* update_surface screen; *)
    (* flip screen; *)
    if !frames = 1000 then finished := true
  done;
  let after = Timer.get_ticks () in
    Sdl.quit();
    if after > before then
      let diff = (float after) -. (float before) in
	Printf.printf "%2.2f frames per second\n" 
	  ((float !frames) *. 1000. /. diff)
  
  
let _ = Printexc.catch main ()

  
    
